@@ -1,4 +1,3 @@
-import sys
 import inspect
 from django.db import models
 from django.conf import settings
@@ -9,6 +8,7 @@ from django.utils.datastructures import MultiValueDict
 
 __all__ = ('ModelTree',)
 
+
 MODELTREE_DEFAULT_ALIAS = 'default'
 
 
@@ -417,7 +417,7 @@ class ModelTree(object):
             # route has been defined with a specific field required
             tup = (field.model, field.rel.to)
             # skip if not the correct field
-            if not self._routes.has_key(tup) or self._routes.get(tup) is field:
+            if tup not in self._routes or self._routes.get(tup) is field:
                 return field
 
     def _filter_related_one2one(self, rel):
@@ -431,7 +431,7 @@ class ModelTree(object):
             # route has been defined with a specific field required
             tup = (rel.model, field.model)
             # skip if not the correct field
-            if not self._routes.has_key(tup) or self._routes.get(tup) is field:
+            if tup not in self._routes or self._routes.get(tup) is field:
                 return rel
 
     def _filter_fk(self, field):
@@ -444,7 +444,7 @@ class ModelTree(object):
             # route has been defined with a specific field required
             tup = (field.model, field.rel.to)
             # skip if not the correct field
-            if not self._routes.has_key(tup) or self._routes.get(tup) is field:
+            if tup not in self._routes or self._routes.get(tup) is field:
                 return field
 
     def _filter_related_fk(self, rel):
@@ -458,7 +458,7 @@ class ModelTree(object):
             # route has been defined with a specific field required
             tup = (rel.model, field.model)
             # skip if not the correct field
-            if not self._routes.has_key(tup) or self._routes.get(tup) is field:
+            if tup not in self._routes or self._routes.get(tup) is field:
                 return rel
 
     def _filter_m2m(self, field):
@@ -471,7 +471,7 @@ class ModelTree(object):
             # route has been defined with a specific field required
             tup = (field.model, field.rel.to)
             # skip if not the correct field
-            if not self._routes.has_key(tup) or self._routes.get(tup) is field:
+            if tup not in self._routes or self._routes.get(tup) is field:
                 return field
 
     def _filter_related_m2m(self, rel):
@@ -485,7 +485,7 @@ class ModelTree(object):
             # route has been defined with a specific field required
             tup = (rel.model, field.model)
             # given this route, check if this is the correct field to JOIN on.
-            if not self._routes.has_key(tup) or self._routes.get(tup) is field:
+            if tup not in self._routes or self._routes.get(tup) is field:
                 return rel
 
     def _add_node(self, parent, model, relation, reverse, related_name,
@@ -496,20 +496,25 @@ class ModelTree(object):
 
         Conditions in which the node will fail to be added:
 
+            - a reverse relationship is blocked via the '+'
             - the model is excluded completely
             - the model is going back the same path it came from
             - the model is circling back to the root_model
             - the model does not come from an explicitly declared parent model
         """
+        # Reverse relationships
+        if reverse and '+' in related_name:
+            return
+
         exclude = set(self.exclude + [parent.parent_model, self.root_model])
 
         # ignore excluded models and prevent circular paths
         if model in exclude:
             return
 
-        # if a route exists, only allow the model to be added if coming from the
-        # specified parent.model
-        if self._tos.has_key(model) and self._tos.get(model) is not parent.model:
+        # if a route exists, only allow the model to be added if coming from
+        # the specified parent.model
+        if model in self._tos and self._tos.get(model) is not parent.model:
             return
 
         node_hash = self._nodes.get(model, None)
@@ -770,20 +775,6 @@ class ModelTree(object):
         "Returns a QuerySet relative to the `root_model`."
         return self.root_model._default_manager.get_query_set()
 
-    def print_path(self, node=None, depth=0):
-        "Traverses the entire tree and prints a hierarchical view to stdout."
-        if node is None:
-            node = self.root_node
-
-        if node:
-            sys.stdout.write('.' * depth * 4)
-            sys.stdout.write(' {0}\n'.format(node.model_name))
-
-        if node.children:
-            depth += 1
-            for x in node.children:
-                self.print_path(x, depth)
-
 
 class LazyModelTrees(object):
     "Lazily evaluates `ModelTree` instances defined in settings."
@@ -795,21 +786,32 @@ class LazyModelTrees(object):
         if alias is None:
             return self.default
 
+        # Echo back an existing modeltree
         if isinstance(alias, ModelTree):
             return alias
 
+        # Already defined
+        if alias in self._modeltrees:
+            return self._modeltrees[alias]
+
+        # Model class
         if inspect.isclass(alias) and issubclass(alias, models.Model):
             return self.create(alias)
 
-        # determine the modeltree instance this should be constructed
-        # relative to
-        if alias not in self._modeltrees:
-            try:
-                kwargs = self.modeltrees[alias]
-            except KeyError:
-                raise ImproperlyConfigured('No modeltree settings defined for "{0}"'.format(alias))
+        # Qualified app.model label
+        if isinstance(alias, basestring) and '.' in alias:
+            app_name, model_name = alias.split('.', 1)
+            model = models.get_model(app_name, model_name)
+            if model is not None:
+                return self.create(model)
 
-            self._modeltrees[alias] = ModelTree(**kwargs)
+        # Assume alias is in the `MODELTREES` setting
+        try:
+            kwargs = self.modeltrees[alias]
+        except KeyError:
+            raise ImproperlyConfigured('No modeltree settings defined for "{0}"'.format(alias))
+
+        self._modeltrees[alias] = ModelTree(**kwargs)
         return self._modeltrees[alias]
 
     @property
